-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Copyright 2016, Knowledge Resources GmbH
-- http://www.knowres.ch
--
-- FILE:        top_level.vhd
-- MODULE:      top_level
-- TITLE:       KRC3701 Template - Top Level 
-------------------------------------------------------------------------------
-- DESCRIPTION: This is a default Top Level module for new designs using the
--              KRC3701 carrier. It includes a correctly configured processing
--              system, which provides a 100MHz and a 50MHz clock by default.
--              It has all standard IO ports predefined and is dependent on the 
--              krc3701_pkg.
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

library UNISIM;
use UNISIM.VComponents.all;

use work.krc3701_pkg.all;

entity top_level is
	Port (
	-- I/O Groups
		X1_1 : inout iogroup_t;
		X1_2 : inout iogroup_t;
		X2_1 : inout iogroup_t;
		X2_2 : inout iogroup_t;
		
		X1_MGT0 : inout mgtgroup0_t;
		X1_MGT1 : inout mgtgroup1_t;

		LED : out STD_LOGIC_VECTOR (3 downto 0);

	-- PS DDR Connections
		DDR_addr : inout STD_LOGIC_VECTOR ( 14 downto 0 );
		DDR_ba : inout STD_LOGIC_VECTOR ( 2 downto 0 );
		DDR_cas_n : inout STD_LOGIC;
		DDR_ck_n : inout STD_LOGIC;
		DDR_ck_p : inout STD_LOGIC;
		DDR_cke : inout STD_LOGIC;
		DDR_cs_n : inout STD_LOGIC;
		DDR_dm : inout STD_LOGIC_VECTOR ( 3 downto 0 );
		DDR_dq : inout STD_LOGIC_VECTOR ( 31 downto 0 );
		DDR_dqs_n : inout STD_LOGIC_VECTOR ( 3 downto 0 );
		DDR_dqs_p : inout STD_LOGIC_VECTOR ( 3 downto 0 );
		DDR_odt : inout STD_LOGIC;
		DDR_ras_n : inout STD_LOGIC;
		DDR_reset_n : inout STD_LOGIC;
		DDR_we_n : inout STD_LOGIC;
		
	-- PS FIXED Connections
		FIXED_IO_ddr_vrn : inout STD_LOGIC;
		FIXED_IO_ddr_vrp : inout STD_LOGIC;
		FIXED_IO_mio : inout STD_LOGIC_VECTOR ( 53 downto 0 );
		FIXED_IO_ps_clk : inout STD_LOGIC;
		FIXED_IO_ps_porb : inout STD_LOGIC;
		FIXED_IO_ps_srstb : inout STD_LOGIC
	);
end top_level;

architecture Behavioral of top_level is

component block_design_wrapper is
	port (
		DDR_addr : inout STD_LOGIC_VECTOR ( 14 downto 0 );
		DDR_ba : inout STD_LOGIC_VECTOR ( 2 downto 0 );
		DDR_cas_n : inout STD_LOGIC;
		DDR_ck_n : inout STD_LOGIC;
		DDR_ck_p : inout STD_LOGIC;
		DDR_cke : inout STD_LOGIC;
		DDR_cs_n : inout STD_LOGIC;
		DDR_dm : inout STD_LOGIC_VECTOR ( 3 downto 0 );
		DDR_dq : inout STD_LOGIC_VECTOR ( 31 downto 0 );
		DDR_dqs_n : inout STD_LOGIC_VECTOR ( 3 downto 0 );
		DDR_dqs_p : inout STD_LOGIC_VECTOR ( 3 downto 0 );
		DDR_odt : inout STD_LOGIC;
		DDR_ras_n : inout STD_LOGIC;
		DDR_reset_n : inout STD_LOGIC;
		DDR_we_n : inout STD_LOGIC;
		FCLK_CLK0 : out STD_LOGIC;
		FCLK_CLK1 : out STD_LOGIC;
		FCLK_RESET0_N : out STD_LOGIC;
		FIXED_IO_ddr_vrn : inout STD_LOGIC;
		FIXED_IO_ddr_vrp : inout STD_LOGIC;
		FIXED_IO_mio : inout STD_LOGIC_VECTOR ( 53 downto 0 );
		FIXED_IO_ps_clk : inout STD_LOGIC;
		FIXED_IO_ps_porb : inout STD_LOGIC;
		FIXED_IO_ps_srstb : inout STD_LOGIC
	);
end component block_design_wrapper;

-- Clocks and active-low Reset signals from the processing system, by default FCLK_CLK0 is at 100MHz and FCLK_CLK1 is at 50MHz.
-- These values can be changed by re-configuring the Processing System in the block diagram.
signal	FCLK_CLK0 : STD_LOGIC;
signal	FCLK_CLK1 : STD_LOGIC;

signal	FCLK_RESET0_N : STD_LOGIC;

signal	USB_HUB_RESET : STD_LOGIC;

-- RGB LED Signals on KRC3701
signal	RGB0_R, RGB0_G, RGB0_B : STD_LOGIC := '0';
signal	RGB1_G, RGB1_B : STD_LOGIC := '0';
signal	RGB2_G, RGB2_B : STD_LOGIC := '0';

-- I2C Bus Signals
signal	I2C_SCL_IN, I2C_SDA_IN : STD_LOGIC;
signal	I2C_SCL_OUT, I2C_SDA_OUT : STD_LOGIC := '1';

-- PL UART Signals
signal PL_UART_RX : STD_LOGIC;
signal PL_UART_TX : STD_LOGIC;

-- Test Point Signals TP47 downto TP40
signal TP : STD_LOGIC_VECTOR(7 DOWNTO 0);

begin

block_design : block_design_wrapper
	port map(
		DDR_addr => DDR_addr,
		DDR_ba => DDR_ba,
		DDR_cas_n => DDR_cas_n,
		DDR_ck_n => DDR_ck_n,
		DDR_ck_p => DDR_ck_p,
		DDR_cke => DDR_cke,
		DDR_cs_n => DDR_cs_n,
		DDR_dm => DDR_dm,
		DDR_dq => DDR_dq,
		DDR_dqs_n => DDR_dqs_n,
		DDR_dqs_p => DDR_dqs_p,
		DDR_odt => DDR_odt,
		DDR_ras_n => DDR_ras_n,
		DDR_reset_n => DDR_reset_n,
		DDR_we_n => DDR_we_n,
		FCLK_CLK0 => FCLK_CLK0,
		FCLK_CLK1 => FCLK_CLK1,
		FCLK_RESET0_N => FCLK_RESET0_N,
		FIXED_IO_ddr_vrn => FIXED_IO_ddr_vrn,
		FIXED_IO_ddr_vrp => FIXED_IO_ddr_vrp,
		FIXED_IO_mio => FIXED_IO_mio,
		FIXED_IO_ps_clk => FIXED_IO_ps_clk,
		FIXED_IO_ps_porb => FIXED_IO_ps_porb,
		FIXED_IO_ps_srstb => FIXED_IO_ps_srstb
	);
	
-- Instantiate the USB Hub Reset Module
usb_hub_reset_inst: krc3701_usb_hub_reset
	generic map(
		NBITS => 8,
		CNT_MAX => 256
	)
	port map(
		clk => FCLK_CLK1,
		USB_HUB_RESET => USB_HUB_RESET
	);
	
-- Connect the signal to Connector Port
X2_2.t3(7) <= USB_HUB_RESET;

-- Connect the RGB LED Signals
X2_2.t3(0) <= RGB0_R;
X2_2.t3(1) <= RGB0_G;
X2_2.t3(2) <= RGB0_B;
X2_2.t3(3) <= RGB1_G;
X2_2.t3(4) <= RGB1_B;
X2_2.t3(5) <= RGB2_G;
X2_2.t3(6) <= RGB2_B;

-- Connect the PL UART Signals
PL_UART_RX <= X2_1.t3(11);
X2_1.t3(10) <= PL_UART_TX;

-- Connect the TP Signals
X2_1.t3(7 downto 0) <= TP(7 downto 0);

-- Instantiate Tri-State Buffers for I2C and connect Signals
IOBUF_SCL : IOBUF
	generic map (
		DRIVE => 12,
		IOSTANDARD => "LVCMOS18",
		SLEW => "SLOW")
	port map (
		O => I2C_SCL_IN, 	-- Buffer output
		IO => X2_1.t3(8), 	-- Buffer inout port (connect directly to top-level port)
		I => '0', 	-- Buffer input
		T => I2C_SCL_OUT 	-- 3-state enable input, high=input, low=output 
	);
IOBUF_SDA : IOBUF
	generic map (
		DRIVE => 12,
		IOSTANDARD => "LVCMOS18",
		SLEW => "SLOW")
	port map (
		O => I2C_SDA_IN, 	-- Buffer output
		IO => X2_1.t3(9), 	-- Buffer inout port (connect directly to top-level port)
		I => '0', 	-- Buffer input
		T => I2C_SCL_OUT 	-- 3-state enable input, high=input, low=output 
	);

end Behavioral;
